import logging
from paircd.reply import ERR_USERSDONTMATCH, MODE, RPL_CHANNELMODEIS, RPL_UMODEIS

from paircd.client import Client
from paircd.command_handler import CommandHandler
from paircd.message import Message
from paircd.server import Server


class ModeHandler(CommandHandler):
    def __init__(self) -> None:
        super().__init__("MODE")

    async def handle(self, server: Server, client: Client, msg: Message) -> None:
        name = msg.args[0]
        if not name.startswith("#"):
            if name == client.nickname:
                settings = client.get_mode_settings()
                client.write_message(RPL_UMODEIS(client.nickname, settings))
            else:
                client.write_message(ERR_USERSDONTMATCH(client.nickname))
            return

        channel = server.get_channel_by_name(name)
        modes = "".join(channel.get_modes())
        if modes != "":
            modes = f"+{modes}"

        client.log("TODO: implement channel modes", level=logging.WARN)
        client.write_message(MODE(name, modes))
        client.write_message(RPL_CHANNELMODEIS(client.nickname, name, modes, ""))
